<?php if (!defined('VB_ENTRY')) die('Access denied.');
require_once(DIR . '/vb/search/itemindexer.php');

/**
 * @package vbdbsearch
 * @author Pritesh Shah, vBulletin Development Team
 */

class vBSphinxSearch_Indexer extends vB_Search_ItemIndexer
{


	/**
	 * Enter description here...
	 *
	 * @param unknown_type $fields
	 *   this is an array of field-value pairs. We need to have at least
	 *   primaryid and contenttype. The id may come as primaryid or id.
	 */

	//let's store some information that lets us go from contentid to type:

	public function index($fields)
	{
		global $vbulletin;
		$db = $vbulletin->db;

		$string_fields = array('username');
		$int_fields = array('contenttypeid', 'primaryid');
		//$keyword_fields = array('title', 'keywordtext');
		$dbfields = array();
		
		if (array_key_exists('id',$fields) AND !(array_key_exists('primaryid',$fields)))
		{
			$fields['primaryid']= $fields['id'];
			unset($fields['id']);
		}

		if (!array_key_exists('primaryid', $fields) OR !array_key_exists('contenttypeid', $fields))
		{
			return false;
		}

		foreach ($int_fields as $key)
		{
			if (isset($fields[$key]))
			{
				$dbfields[$key] = intval($fields[$key]);
			}
		}
		
		
		$db->query_write($this->make_save_query('vbsphinxsearch_queue', $dbfields));		
		
		//$db->query_write($this->make_save_query('searchcore', $dbfields));


		//efficency hack.
		//Store the group fileds separately to avoid having to reindex multiple items
		//when the group data changes.
		//
		//The title is also imporant for searching.  The original implementation naively used
		//the thread title for the post title.  With so many indentical values in the full text
		//index performance proved to be poor.  Spliting the text into

		//$this->save_group_data($fields);

	}

	private function save_group_data($fields)
	{
		global $vbulletin;
		$db = $vbulletin->db;

		$group_int_fields = array (
			'groupcontenttypeid' => 'contenttypeid',
			'groupid' => 'primaryid',			
		);

		
		$dbfields = array();
		

		foreach ($group_int_fields as $key => $dbkey)
		{
			if (isset($fields[$key]))
			{
				$dbfields[$dbkey] = intval($fields[$key]);
			}
		}


		if (count($dbfields) AND isset($fields['groupid']) AND $fields['groupid'])
		{
			
			$db->query_write($this->make_save_query('vbsphinxsearch_queue', $dbfields));
			
		}
	}

	private function make_save_query($table, $dbfields)
	{
		//There could be a substantial performance penalty for using
		// REPLACE INTO. Therefore we will use a non-standard
		// MySQL enhancement as shown.
		$sql = "INSERT INTO " . TABLE_PREFIX . "$table (" . implode(', ', array_keys($dbfields)) . ")
			VALUES ( " . implode(', ', $dbfields) . " )
			ON DUPLICATE KEY UPDATE ";
		$first = true;
		
		foreach ($dbfields as $key => $value)
		{
			if ($first)
			{
				$sql .= " $key = VALUES($key)";
				$first = false;
			}
			else
			{
				$sql .= ", $key = VALUES($key)";
			}
		}

		return $sql;
	}


	/**
	 * Delete the record from the index
	 *
	 * If its the last record in its group, then nuke the group record too.
	 *
	 * @param unknown_type $contenttype
	 * @param unknown_type $id
	 */
	public function delete($contenttypeid, $id)
	{
		$safe_contenttypeid = intval($contenttypeid);
		$safe_id = intval($id);
		$fields = array();
		$fields['primaryid'] = $safe_id;
		$fields['contenttypeid'] = $safe_contenttypeid;
		$this->index($fields);	
	}


	public function merge_group($groupcontenttypeid, $oldid, $newid)
	{
		global $vbulletin;

		$safe_groupcontenttypeid = intval($groupcontenttypeid);
		$safe_oldid = intval($oldid);
		$safe_newid = intval($newid);
		
		$this->delete_group($safe_groupcontenttypeid, $safe_oldid);
		/*
		$fields['groupcontenttypeid'] = $safe_groupcontenttypeid;
		$fields['groupid'] = $safe_oldid;
		$this->save_group_data($fields);
		*/
		$fields['groupcontenttypeid'] = $safe_groupcontenttypeid;
		$fields['groupid'] = $safe_newid;
		$this->save_group_data($fields);
		
	}

	public function group_data_change($groupdata)
	{
		$this->save_group_data($groupdata);
	}

	public function delete_group($groupcontenttypeid, $groupid)
	{
		global $vbulletin;

		$safe_groupcontenttypeid = intval($groupcontenttypeid);
		$safe_groupid = intval($groupid);
		
		$fields = array();
		$fields['groupid'] = $safe_groupid;
		$fields['groupcontenttypeid'] = $safe_groupcontenttypeid;
		$this->save_group_data($fields);		
	}

	/**
	 * Do any necessary manipulation of the keyword text
	 *
	 * do any keywords related filtering -- such as handling non whitespace
	 * tokenization or other manipulation
	 *
	 * Primarily intended for subclasses for odd language sets
	 *
	 * @param string $text
	 * @return string
	 */
	protected function filter_keywords($text)
	{
		return $text;
	}
}
