<?php if (!defined('VB_ENTRY')) die('Access denied.');

/**
 * @package vBulletin
 * @subpackage Search
 * @author Pritesh Shah, vBulletin Development Team
 */

require_once (DIR . '/vb/search/type.php');
require_once (DIR . '/vb/legacy/post.php');
require_once (DIR . '/packages/vbforum/search/result/post.php');
require_once (DIR . '/packages/vbforum/search/result/thread.php');

/**
* There is a type file for each search type. This is the one for posts
*
* @package vBulletin
* @subpackage Search
*/
class vBForum_Search_Type_Post extends vB_Search_Type
{
// ###################### Start fetch_validated_list ######################
/**
* vBForum_Search_Type_Post::fetch_validated_list()
* When displaying results we get passed a list of id's. This
* function determines which are viewable by the user.
*
* @param object $user
* @param array $ids : the postid's returned from a search
* @param array $gids : the group id's for the posts
* @return array (array of viewable posts, array of rejected posts)
*/
	public function fetch_validated_list($user, $ids, $gids)
	{
		global $vbulletin;
		$map = array();
		foreach ($ids AS $i => $id)
		{
			$map[$gids[$i]][] = $id;
		}

		$threads = vB_Legacy_Thread::create_array(array_unique($gids));
		$rejected_groups = array();
		foreach ($threads as $thread)
		{
			if (!$thread->can_search($user))
			{
				$rejected_groups[] = $thread->get_field('threadid');
				unset($map[$thread->get_field('threadid')]);
			}
		}

		$searchids = array();
		foreach ($map as $gid => $value)
		{
			$searchids = array_merge($searchids, $value);
		}

		$list = array();

		if (count($searchids))
		{
			$set = $vbulletin->db->query("
				SELECT p.*
				FROM " . TABLE_PREFIX . "post AS p
				WHERE p.postid IN (" . implode(',', $searchids) . ")
			");

			while ($row = $vbulletin->db->fetch_array($set))
			{
				$item = vBForum_Search_Result_Post::create_from_object(
					vB_Legacy_Post::create_from_record($row, $threads[$row['threadid']]));

				if ($item->can_search($user))
				{
					$list[$row['postid']] = $item;
				}
				else
				{
					$list[$row['postid']] = false;
				}
			}
		}
		return array('list' => $list, 'groups_rejected' => $rejected_groups);
	}

// ###################### Start prepare_render ######################
/**
* vBForum_Search_Type_Post::prepare_render()
* This function does setup preparatory to  rendering the search resuls.
* In this case that means checking moderation options for this user.
*
* @param object $user
* @param object $results
* @return nothing
*/
	public function prepare_render($user, $results)
	{
		foreach ($results AS $result)
		{
			$forumid = $result->get_post()->get_thread()->get_field('forumid');

			$this->mod_rights['managethread'] = ($this->mod_rights['managethread'] OR
				$user->canModerateForum($item['forumid'], 'canmanagethreads'));

			$this->mod_rights['managepost'] = ($this->mod_rights['managepost'] OR
				($user->canModerateForum($item['forumid'], 'candeleteposts') OR
					$user->canModerateForum($item['forumid'], 'canremoveposts')));

			$this->mod_rights['approvepost'] = ($this->mod_rights['approvepost'] OR
				$user->canModerateForum($item['forumid'], 'canmoderateposts'));

			$this->mod_rights['approveattachment'] = ($this->mod_rights['approveattachment'] OR
				$user->canModerateForum($item['forumid'], 'canmoderateattachments'));
		}
	}

// ###################### Start get_display_name ######################
/**
* vBForum_Search_Type_Post::get_display_name()
* Each search type has some responsibilities, one of which is to give
* its display name.
*
* @return string
*/
	public function get_display_name()
	{
		return $GLOBALS['vbphrase']['post'];
	}

// ###################### Start create_item ######################
/**
* vBForum_Search_Type_Post::create_item()
* This is how the type objects are created
*
* @param integer $id
* @return vBForum_Search_Type_Post object
*/
	public function create_item($id)
	{
		return vBForum_Search_Result_Post::create($id);
	}

// ###################### Start additional_pref_defaults ######################
/**
* Each search type has some responsibilities, one of which is to tell
* what the default search preferences are.
* vBForum_Search_Type_Post::additional_pref_defaults()
*
* @return array
*/
	public function additional_pref_defaults()
	{
		return array(
			'childforums' => 1,
			'replyless'   => 0,
			'replylimit'  => 0,
			'query'       => '',
			'forumchoice' => 0,
			'prefixchoice'=> 0,
			'titleonly'   => 0,
			'nocache'     => '',
			'searchuser'  => '',
			'starteronly' => 0,
			'showposts'   => 0,
			'exactname'   => '',
			'searchdate'  => 0,
			'beforeafter' => 0,
			'sortby'      => 'dateline',
			'order' 	     => 'descending',
			'tag'         => '',
			'showposts'	  => 0);
	}

// ###################### Start can_group ######################
/**
* vBForum_Search_Type_Post::can_group()
* Each search type has some responsibilities, one of which is to tell
* whether it is groupable- Forums, for example are not, but posts are.
* They are naturally grouped by thread.
*
* @return
*/
	public function can_group()
	{
		return true;
	}

// ###################### Start group_by_default ######################
/**
* vBForum_Search_Type_Post::group_by_default()
* Each search type has some responsibilities, one of which is to tell
* whether it is grouped by default
*
* @return
*/
	public function group_by_default()
	{
		return true;
	}

// ###################### Start listUi ######################
/**
 * vBForum_Search_Type_Post::listUi()
 *This function generates the search elements for the user to search for posts
 *
 * @return $html: complete html for the search elements
 */
	public function listUi($prefs)
	{
		global $vbulletin;
		$template = vB_Template::create('search_input_post');
		$template->register('securitytoken', $vbulletin->userinfo['securitytoken']);
		$template->register('contenttypeid', vB_Search_Core::get_instance()->get_contenttypeid('vBForum', 'Post'));
		$template->register('search_forum', $this->showForumSelect('forumchoice', 'class="bginput"', $prefs['forumchoice']));
		$template->register('search_prefix', vB_Search_Searchtools::showPrefixes('prefixchoice', 'class="bginput"', $prefs['prefixchoice']));
		$template->register('tag_cloud', fetch_tagcloud('selectlist', $prefs['tags']));

		if ($vbulletin->options['threadtagging'] == 1 AND $vbulletin->options['tagcloud_searchcloud'] == 1)
		{
			$tag_cloud = fetch_tagcloud('search');

			if ($tag_cloud)
			{
				$template->register('tag_cloud', $tag_cloud);
			}
		}

		$this->setPrefs($template, $prefs,  array(
			'select'=> array('sortby', 'order', 'forumchoice',
				'prefixchoice', 'titleonly', 'starteronly', 'searchdate',
				'beforeafter'),
			'cb' => array('childforums', 'exactname', 'nocache'),
		 	'value' => array('query', 'searchuser', 'tag'),
		 	'rb' => array('showposts')
		 	) );
		vB_Search_Searchtools::searchIntroRegisterHumanVerify($template);
		return $template->render();
	}
// ###################### Start showForumSelect ######################
/**
 * vBForum_Search_Type_Post::showForumSelect()
 * This function generates the select scrolling list for forums, use in search for posts
 *
 * @param string $name : name for the select element
 * @param string $style_string : something like "style=XXXX" or "class=XXX". Or empty
 * @return $html: complete html for the search interface
 */
	private function showForumSelect($name, $style_string,
		$forumchoice=array())
	{
		global $vbulletin, $vbphrase, $show;

		//this will fill out $searchforumids as well as set the depth param in $vbulletin->forumcache
		global $searchforumids;
		fetch_search_forumids_array();


		$options = "";
		foreach ($searchforumids AS $forumid)
		{
			$forum = & $vbulletin->forumcache["$forumid"];

			if (trim($forum['link']))
			{
				continue;
			}

			$optionvalue = $forumid;
			$optiontitle = "$forum[depthmark] $forum[title_clean]";

			if ($vbulletin->options['fulltextsearch'] AND
				!($vbulletin->userinfo['forumpermissions'][$forumid] & $vbulletin->bf_ugp_forumpermissions['canviewthreads']))
			{
				$optiontitle .= '*';
				$show['cantsearchposts'] = true;
			}

			$optionselected = '';

			if ($forumchoice AND in_array($forumid, $forumchoice))
			{
				$optionselected = 'selected="selected"';
				$haveforum = true;
			}

			$options .= render_option_template($optiontitle, $forumid, $optionselected,
				'fjdpth' . min(4, $forum['depth']));
		}

		$select = "<select name=\"" .$name."[]\" multiple=\"multiple\" size=\"4\" $style_string>\n" .
					render_option_template($vbphrase['search_all_open_forums'], '',
						$haveforum ? '' : 'selected="selected"') .
					render_option_template($vbphrase['search_subscribed_forums'], 'subscribed') .
					$options .
				 	"</select>\r";
		return $select;
	}


// ###################### Start get_inlinemod_options ######################
/**
* vBForum_Search_Type_Post::get_inlinemod_options()
* This tells inline moderation what options are applicable to posts
*
* @return array of options
*/
	public function get_inlinemod_options()
	{
		global $vbphrase, $show;
		$options = array();

		$mod_options = array();

		if ($this->mod_rights['managepost'])
		{
			$mod_options[$vbphrase['delete_posts']] = 'deleteposts';
			$mod_options[$vbphrase['undelete_posts']] = 'undeleteposts';
		}

		if ($this->mod_rights['deletethread'] OR $this->mod_rights['movethread'])
		{
			$mod_options[$vbphrase['delete_threads_as_spam']] = 'spamthread';
		}

		if ($this->mod_rights['approvepost'])
		{
			$mod_options[$vbphrase['approve_posts']] = 'approveposts';
			$mod_options[$vbphrase['unapprove_posts']] = 'unapproveposts';
		}

		if ($this->mod_rights['approveattachment'])
		{
			$mod_options[$vbphrase['approve_attachments']] = 'approveattachments';
			$mod_options[$vbphrase['unapprove_attachments']] = 'unapproveattachments';
		}

		if ($this->mod_rights['managethread'])
		{
			$mod_options[$vbphrase['move_posts']] = 'moveposts';
			$mod_options[$vbphrase['merge_posts']] = 'mergeposts';
			$mod_options[$vbphrase['copy_posts']] = 'copyposts';
		}

		//if we have any mod options then we add the rest
		if ($mod_options)
		{
			$options[$vbphrase['option']] = $mod_options;
			$basic_options = array();
			$basic_options[$vbphrase['view_selected_posts']] = 'viewpost';
			$basic_options[$vbphrase['clear_post_list']] = 'clearpost';
			$options ["____________________"] = $basic_options;
		}
		return $options;
	}

// ###################### Start get_inlinemod_type ######################
/**
* vBForum_Search_Type_Post::get_inlinemod_type()
* Tell what type of object this is for inline moderation
*
* @return string
*/
	public function get_inlinemod_type()
	{
		return 'post';
	}

	public function get_inlinemod_action()
	{
		return 'inlinemod.php';
	}

	/**
	 * vBForum_Search_Type_Post::get_query()
	 * This function generates the additional query information
	 * required to return the correct values
	 *
	 * @return array :('join' => extra join code,
	 *   'where' => extra filter code, 'sort'=>sortby)
	 */
	public function get_query()	{
		$join = '';
		$where = array();
		print "WE R HERE";
		if (isset($this->forumchoice))
		{
			$join .= sprintf(self::$thread_join, TABLE_PREFIX,
				vB_Search_Core::get_instance()->get_contenttypeid("vBForum", "Post") )
				. sprintf(self::$forum_thread_join, TABLE_PREFIX );

			if (!is_array($this->forumchoice) )
			{
				$this->forumchoice = array($this->forumchoice);
			}
			$forumids = array();
			foreach ($this->forumchoice as $forumid)
			{
				$forumids[]= $forumid;

				if ($this->childforums)
				{
					$forumids[]= fetch_child_forums($forumid);
				}
			}
			$where [] = 'forum.forumid In (' . implode(', ', $forumids) . ')';
		}

		if (isset($this->prefixchoice))
		{
			if (! isset($this->forumchoice))
			{
				$join .= sprintf(self::$thread_join, TABLE_PREFIX,
					vB_Search_Core::get_instance()->get_contenttypeid("vBForum", "Post") )
					. sprintf(self::$forum_thread_join, TABLE_PREFIX );
			}

			if (!is_array($this->prefixchoice))
			{
				$this->prefixchoice = array($this->prefixchoice);
			}
			$prefixes = array();
			foreach($this->prefixchoice as $key => $value)
			{
				$prefixes[$key] = "'" . $value . "'";
			}
			$where[] = 'thread.prefixid in (' . implode(', ', $prefixes) . ') ';
		}
	
		//we return the results differently. Join is just a string, because
		// we combine it with the primary join(s) as a string.
		//The 'where' clause requires slightly more complex handling, so it's
		// better to keep it as an array. In the calling class (currently
		// coresearchcontroller) we merge the array and then do an implode.

		if ($join == '' or count($where) == 0)
		{
			return null;
		}
		return array('join' => $join,
	 		'where' => $where, null );
	}

	/**
	 * vBForum_Search_Type_Post::get_filters()
	 * this function returns an array of all the currently set filters
	 *
	 * @return array
	 */
	public function get_filters()
	{
		$result = array();
		if (isset($this->prefixchoice))
		{
			$result['prefixchoice'] = array(
				vB_Search_Core::OP_EQ => $this->prefixchoice);
		}

		if (isset($this->forumchoice))
		{
			$result['forumchoice'] = array(
				vB_Search_Core::OP_EQ => $this->forumchoice);
		}

		return $result;
	}

	/**
	 * vBForum_Search_Type_Post::add_filter()
	 * This function is called to add a filter for a form field.
	 * It is only called if we passed filter information in
	 * add_advanced_search_filters. This function should also register the
	 * display strings with the criteria object
	 *
	 * @param string $field
	 * @param mixed $value : it could be a single value or an array. The
	 *   child will need to know what to do
	 * @return no return
	 */
	public function add_filter($field, $value)
	{
		global $vbphrase, $vbulletin;

		if ($field == 'forumchoice' AND (count($value) > 0 OR intval($value) > 0) )
		{

			if (! is_array($value))
			{
				$value = array($value);
			}

			if (in_array(' ',$value) or in_array('',$value))
			{
				unset($this->form_vars[$field]);
				return;
			}
			$this->forumchoice = $value;
			$display_field = $vbphrase['forum'];


			if ($rst = $vbulletin->db->query_read("SELECT title from " . TABLE_PREFIX
				. "forum where forumid in (" . implode(', ', $this->forumchoice)
				. ")") )
			{
				$forums = array();
				while ($row = $vbulletin->db->fetch_row($rst))
				{
					$forums[] = $row[0];
				}
			}
			$this->criteria->add_display_strings($field,
				"$display_field :"
				. implode( ', ' , $forums));
		}
		else if ($field == 'prefixchoice' AND (count($value) > 0 OR intval($value) > 0) )
		{

			if (! is_array($value))
			{
				$value = array($value);
			}

			if (in_array(' ',$value) or in_array('',$value))
			{
				unset($this->form_vars[$field]);
				return;
			}
			$this->prefixchoice = $value;
			$display_field = $vbphrase['prefix'];
			$prefixes = array();

			foreach ($value as $prefixid)
			{
				$prefixes[] =
						$vbphrase["prefix_{$prefixid}_title_rich"];
			}
			$this->criteria->add_display_strings($field,
				"$display_field :"
				. implode( ', ' , $prefixes));
		}
		else if ($field == 'childforums' AND intval($value) > 0)
		{
			$this->childforums = $value;
			$this->criteria->add_display_strings('child_forums',
				$vbphrase['and_child_forums']);
		}
	}

	protected $prefixchoice;
	protected $forumchoice;
	protected $childforums;

	protected $package = "vBForum";
	protected $class = "Post";
	protected $group_package = "vBForum";
	protected $group_class = "Thread";
	protected $type_globals = array (
		'showposts'      => TYPE_INT,
		'forumchoice'	  => TYPE_ARRAY,
		'starteronly'    => TYPE_INT,
		'prefixchoice'	  => TYPE_ARRAY,
		'childforums'	  => TYPE_BOOL);
	private static $thread_join =
	  " INNER JOIN %sthread AS thread ON (
			searchcore.contenttypeid =%u  AND searchcore.groupid = thread.threadid)";
	private static $forum_thread_join =
		" INNER JOIN %sforum AS forum ON (
			thread.forumid = forum.forumid)
		";

}

